package adlpadmin::config;

use strict;

use Config::IniFiles;


=encoding utf8

=head1 SYNOPSIS

 use adlpadmin::config;

 # Give custom paths for adlpadmin.ini
 my $cfg1 = new adlpadmin::config(
    '/etc/local/lab_adlpadmin.ini',
    '/etc/local/adlpadmin.ini'
    );

 print $cfg1->{'json_url'};
 
 # Use default paths for adlpadmin.ini
 my $cfg2 = new adlpadmin::config;
 
 if ($cfg2->printer_is_addable('smb://server/queue')) {
    print "You may add this printer\n";
    }

=head1 DESCRIPTION

adlpadmin::config is a class that reads adlpadmin.ini configuration file.
The most common use is to read the configuration file settings
directly from the public class variables. There are some methods as well.

=head1 METHODS

=over 4

=item new

The constructor reads the ini files. The parameters are possible filenames
for ini files to read. The files are read beginning from the first parameter.
When a readable file is found it is processed and the later files are not
processed.

If no parameters are given the default array C<('/etc/adlpadmin.ini',
'/etc/adlpadmin/adlpadmin.ini')> is used.

After this the settings are read directly from the class variables.

=back

=cut

sub new {
	my ($class, @ini_files) = @_;
	
	my $self = {};
	
	my $ini_file = undef;

	if (scalar(@ini_files) == 0) {
		# No parameters given, use defaults
		@ini_files = ('/etc/adlpadmin.ini',
			'/etc/adlpadmin/adlpadmin.ini');
		}

	for (my $n=0; $n < scalar(@ini_files); $n++) {
		if (-r $ini_files[$n]) {
			# This file is readable
			$ini_file = $ini_files[$n];
			last;
			}
		}
	
	if ($ini_file) {
		# There is a ini file
		my $cfg = Config::IniFiles->new(
			-file => $ini_file,
			-nocase => 1
			);
		
		($self->{'size_small_x'}, $self->{'size_small_y'})
			= split(/,/, $cfg->val('Window', 'size_small', '300,300'));
		($self->{'size_large_x'}, $self->{'size_large_y'})
			= split(/,/, $cfg->val('Window', 'size_large', '600,600'));
		
		$self->{'ad_method'} = lc($cfg->val('Active Directory', 'AuthMethod'));
		$self->{'ad_simple_domain'} = $cfg->val('Active Directory', 'SimpleDomain', '');
		$self->{'ad_dc'} = $cfg->val('Active Directory', 'DC');
		$self->{'ad_ou'} = join("\t", $cfg->val('Active Directory', 'OU'));
		$self->{'ad_charset'} = $cfg->val('Active Directory', 'Charset', 'utf8');

		$self->{'ppd_pathppd'} = join("\t", $cfg->val('PPD', 'PathPPD'));
		$self->{'ppd_pathmodel'} = $cfg->val('PPD', 'PathModel');
		$self->{'ppd_defaultd'} = $cfg->val('PPD', 'DefaultDriver');
		$self->{'ppd_defaultf'} = $cfg->val('PPD', 'DefaultPPD');
		$self->{'ppd_allowpath'} = join("\t", $cfg->val('PPD', 'AllowPath'));
		
		$self->{'backend'} = $cfg->val('Printing', 'Backend');
		$self->{'cmd_add'} = $cfg->val('Printing', 'Add');
		$self->{'cmd_rem'} = $cfg->val('Printing', 'Remove');
		$self->{'add_charset'} = $cfg->val('Printing', 'Charset', 'utf8');
		$self->{'add_acc_uri'} = join("\t", $cfg->val('Printing', 'AddAcceptUri'));
		$self->{'rem_acc_uri'} = join("\t", $cfg->val('Printing', 'RemovalAcceptUri'));
		$self->{'lpadmin_path'} = $cfg->val('Printing', 'Lpadmin_path');
		$self->{'servername_length'} = $cfg->val('Printing', 'ServerName', 'long');
		
		$self->{'cupsoption_acc'} = join("\t", $cfg->val('Print Settings', 'AllowedCUPSoption'));
		$self->{'allowed_pagesize'} = join("\t", $cfg->val('Print Settings', 'PageSize'));
		$self->{'fallback_pagesize'} = $cfg->val('Print Settings', 'FallbackPageSize');
		$self->{'cupsoption_forced_pagesize'} = $cfg->val('Print Settings', 'ForcedPageSize');
		$self->{'cupsoption_forced_duplex'} = $cfg->val('Print Settings', 'ForcedDuplex');
		}

	bless($self, $class);
	
	return $self;
	}

=item clean

Clean (untaint) strings that are intended for CUPS administrative options. Accepts
strings that do not contain characters ", ', ; and &.

 my $settings = new adlpadmin::config('adlpadmin.ini');
 my $param = 'Media="; rm -fR /;';
 
 print "Param: ".$settings->clean($param)."\n";
 
 Prints ""

=cut

sub clean {
	my ($class, $param) = @_;
	
	if ($param =~ /^([^\"\';\&]*)$/) {
		return $1;
		}
	else {
		return '';
		}
	}

=item printer_is_addable

Returns TRUE if printer URI is among user-addable queues (see
configuration directive C<AddAcceptUri>). Otherwise FALSE.

 my $settings = new adlpadmin::config('adlpadmin.ini');
 
 if (!$settings->printer_is_addable($printer_uri) {
    print "Sorry, you may not add printer with URI $printer_uri\n";
    }

=cut

sub printer_is_addable {
	my ($class, $uri) = @_;
	
	# Default value: not addable
	my $is_addable = undef;
	
	# Go through all accept rules (URI)
	foreach my $this_accept_rule_uri (split(/\t/, $class->{'add_acc_uri'})) {
		if ($uri =~ /$this_accept_rule_uri/) {
			$is_addable = 1;
			}
		}
	
	return $is_addable;
	}
	
=item ppd_is_allowed

Returns TRUE if PPD path is among allowed paths (see configuration
directive C<AllowPath>). Otherwise FALSE.

 my $settings = new adlpadmin::config('adlpadmin.ini');

 if (!$settings->ppd_is_allowed($desired_ppd)) {
    print "Sorry, you may not use PPD at $desired_ppd\n";
    exit(1);
    }

=cut

sub ppd_is_allowed {
	my ($class, $ppd) = @_;
	
	# Default value: not allowed
	my $is_allowed = undef;
	
	# Go through all accept rules (PPD)
	foreach my $this_accept_rule_ppd (split(/\t/, $class->{'ppd_allowpath'})) {
		if ($ppd =~ /$this_accept_rule_ppd/) {
			$is_allowed = 1;
			}
		}
		
	return $is_allowed;
	}

=item cupsoption_is_allowed

Returns TRUE if setting is among CUPS settings that user can set (see
configuration directive C<>). Otherwise FALSE.

 my $settings = new adlpadmin::config('adlpadmin.ini');

 if (!$settings->cupsoption_is_allowed('Media')) {
    print "Sorry, you may not set CUPS option Media\n";
    exit(1);
    }

=cut

sub cupsoption_is_allowed {
	my ($class, $option) = @_;
	
	# Default value: not allowed
	my $is_allowed = undef;
	
	# Go through all accept rules (PPD)
	foreach my $this_accept_rule_option (split(/\t/, $class->{'cupsoption_acc'})) {
		if ($option =~ /$this_accept_rule_option/) {
			$is_allowed = 1;
			}
		}
		
	return $is_allowed;
	}

=item printer_is_removable

Returns TRUE if printer URI is among user-removable queues (see
configuration directive C<RemovalAcceptUri>). Otherwise FALSE.

 my $settings = new adlpadmin::config('adlpadmin.ini');
 
 if ($settings->printer_is_removable($printer_uri) {
    print "OK, you may remove printer at $printer_uri\n";
    }

=cut

sub printer_is_removable {
	my ($class, $uri) = @_;
	
	# Default value: not removable
	my $is_removable = undef;
	
	# Go through all accept rules (URI)
	foreach my $this_accept_rule_uri (split(/\t/, $class->{'rem_acc_uri'})) {
		if ($uri =~ /$this_accept_rule_uri/) {
			$is_removable = 1;
			}
		}
	
	return $is_removable;
	}


=head1 SETTINGS AND VARIABLES

C<adlpadd> reads its configuration settings from C<adlpadmin.ini>. Following
paths are searched in order to find a readable file: C</etc/adlpadmin.ini>,
C</etc/adlpadmin/adlpadmin.ini>.

The C<adlpadmin.ini> is a Windows-style ini file with following sections and
directives:

B<Window>

=over 8

=item size_small (

Window size for small wizard window.

 [Window]
 size_small=300,300
 size_large=600,600

Variables: C<size_small_x>, C<size_small_y>

=back

B<Active Directory>

=over 8

=item AuthMethod

Authentication method used to authenticate to Active Directory. Possible
values are 'simple' (simple authentication, causes a username/password dialog
to the user) and 'gssapi' (uses Kerberos GSSAPI authentication).

 [Active Directory]
 AuthMethod=gssapi
 
Variable: C<ad_method>

=item SimpleDomain

Default domain used to log in with simple authentication. If not defined
no domain part is suggested.

 [Active Directory}
 SimpleDomain=myorg.com

Variable: C<ad_simple_domain>

=item DC

Domain controller to contact for LDAP query. Multiple domain controllers
can be given. For security reasons use always "ldaps" when using simple
authentication.

 [Active Directory]
 DC=ldaps://dc.myorg.fi

Variable: C<ad_dc> (tab-separated list of strings).
 
=item OU

Organisation Unit(s) to look for print queues. Multiple OUs can be
given with multiple OU parameters.

 [Active Directory]
 OU=ou=2008r2,ou=print_servers,dc=MYORG,dc=FI
 OU=ou=2003,ou=print_servers,dc=MYORG,dc=FI

Variable: C<ad_ou> (tab-separated list of strings)

=item Charset

AD LDAP Charset. Any charset supported by perl Encode module (L<http://perldoc.perl.org/Encode.html>)
can be used. Defaults to "utf8".

 [Active Directory]
 Charset=iso-8859-15

Variable: C<ad_charset>

=item JSON_URL

The URL to a HTTP/JSON search engine.

 [Active Directory]
 JSON_URL=https://myserver/find_json.php

Variable: C<json_url>
 
=back

B<PPD>

=over 8

=item PathPPD

Path for PPD files. Files are searched recursively for correct ModelName
parameter. Multiple paths
can be given with multiple PathPPD parameters. Typically you want to specify
C</usr/share/ppd> that contains your PPD files.
Results from C<PathModel> are used prior to C<PathPPD>.
 
 [PPD]
 PathPPD=/usr/share/ppd
 PathPPD=/usr/share/other_ppd

See also C<PathModel> parameter.

Variable: C<ppd_pathppd>

=item PathModel

Equals to CUPS C<lpadmin> directive C<-m>. Searches filenames that equal to
desired driver name. Results from C<PathModel> are used prior to C<PathPPD>.

 [PPD]
 PathModel=/usr/share/cups/model

See also C<PathPPD> parameter.

Variable: C<ppd_pathmodel>

=item DefaultDriver

Default driver to be used name if no printer-specific driver can be found.
You have to set either C<DefaultDriver> or C<DefaultPPD>.

 [PPD]
 DefaultDriver=Generic PostScript Printer

Variable: C<ppd_defaultd>

=item DefaultPPD

Default PPD file to be used if no printer-specific driver can be found.
Specifies full path to fallback PPD file.
You have to set either C<DefaultDriver> or C<DefaultPPD>.

 [PPD]
 DefaultPPD=/usr/share/ppd/cups-included/postscript.ppd

Variable: C<ppd_defaultf>

=item AllowPath

Defines allowed paths for PPD files. Each supplied PPD file is checked
against AllowPath values. The values are regular expressions. This allows root to
control which PPD files users can use in their print queues and allow
users to use only PPD files from the paths that are not user-writeable.

Multiple C<AllowPath> directives can be given.

 [PPD]
 AllowPath=^/usr/share/ppd/
 AllowPath=^/usr/local/share/ppd/
 
Variable: C<ppd_allowpath> (tab-separated list of strings)

=back

B<Printing>

=over 8

=item Backend

Backend to be used when adding printers. The backend string is used
in device-uri (see C<lpadmin> switch C<-v>). The device-uri is 
[Backend]+":"+[AD UNCpath].

 [Printing]
 Backend=adsmb

Variable: C<backend>

=item Add

Defines the script to be used when adding new printers. Following string
substitutions will be made:

 %P  Printer name (e.g. F28-4FLOOR-334)
 %u  Device URI (e.g. smb://server1/f28-4floor-334)
 %d  Description from the AD printer object
 %l  Location from the AD printer object
 %p  PPD path
 %o  CUPS options (options and values will be wrapped inside hyphens)
 
 [Printing]
 Add=/usr/sbin/lpadmin -p "%P" -v "%u" -D "%d" -L "%l" -P "%p" %o

Variable: C<cmd_add>

=item AddAcceptUri

Defines the filters for user-addable printers. Each filter is a regular
expression string. If the printer URI fits to any of the defined strings.
Multiple C<AddAcceptUri> directives can be given.

 [Printing]
 AddAcceptUri=^adsmb\:\/\/

Variable: C<add_acc_uri> (tab-separated list of strings)

=item Remove

Defines the script to be used when deleting printers. Following string
substitutions will be made:

 %P  Printer name (e.g. F28-4FLOOR-334)
 
 [Printing]
 Remove=/usr/sbin/lpadmin -x "%P"
 
Variable: C<cmd_rem>

=item RemovalAcceptUri

Defines the filters for user-removable printers. Each filter is a regular
expression string. If the printer URI fits to any of the defined strings.
Multiple C<RemovalAcceptUri> directives can be given.

 [Printing]
 RemovalAcceptUri=^adsmb\:\/\/
 RemovalAcceptUri=^smb\:\/\/

Variable: C<rem_acc_uri> (tab-separated list of strings)

=item Charset

Charset used when executing Add or Remove command. Edit this if your non-american
characters won't get right in printer description and location fields.

The value should be something that Encode::encode takes as C<ENCODING> parameter
(see L<http://perldoc.perl.org/Encode.html>). Defaults to utf8.

 [Printing]
 Add=/usr/bin/lpadmin -p "%P" -v "%u" -D "%d" -L "%l" -P "%p"
 Charset=iso-8859-15

Variable: C<add_charset>

=item Lpadmin_path

Path for CUPS lpadmin tool. 

 [Printing]
 Lpadmin_path = /usr/sbin/lpadmin
 
Variable: C<lpadmin_path>

=item ServerName

Whetner C<adlpadmin> adds print server with their long (eg. C<prserver.mydomain.org>,
this is the default) or short (e.g. C<prserver>) server names. Legal values are
C<long> and C<short>.

 [Printing]
 ServerName = short

Variable: C<servername_length>

=back

B<Print Settings>

=over 8

=item AllowedCUPSoption

Defines a filter of CUPS options which the user can set while creating a
print queue to the local CUPS server.
Multiple C<AllowedCUPSoption> directives can be given.

 [Print Settings]
 AllowedCUPSoption=Media
 AllowedCUPSoption=Duplex
 AllowedCUPSoption=PageSize

Variable: C<cupsoption_acc> (tab-separated list of strings)

=item PageSize

Defines an accepted page size. C<adlpadd> reads the supported page sizes (C<printMediaSupported>)
from the print queues. Only the sizes that are defined using PageSize are presented
to the user as a possible default page size. The directive is case insensitive
(a3 and A3 are the same). Multiple C<PageSize> directives can be given.

If no PageSize directives are given all values are given to the user to select.

 [Print Settings]
 PageSize=A4
 PageSize=A3
 PageSize=B4

Variable: C<allowed_pagesize> (tab-separated list of strings)

=item FallbackPageSize

Defines a fallback page size value. If no supported pagesizes are retrieved from server
this setting is used. If no FallbackPageSize value is given the default pagesize is not set.

Reasonable values are A4 and Letter.

 [Print Settings]
 FallbackPageSize=A4
 
Variable C<fallback_pagesize>

=item ForcedPageSize

Defines forced default pagesize for all printers that are added.
If this setting is defined the user cannot select default pagesize.
The value must be recognised by CUPS (PPD).

 [Print Settings]
 ForcedPageSize=A4

Variable: C<cupsoption_forced_pagesize>

=item ForcedDuplex

Defines forced default duplex setting for all printers that
are added. If this setting is defined the user cannot select default
duplex value. The value must be recognised by CUPS (PPD). Reasonable
values are "None", "DuplexNoTumble" and "DuplexTumble".

 [Print Settings]
 ForcedDuplex=DuplexNoTumble

Variable: C<cupsoption_forced_duplex>

=back

=cut

1;
	
__END__

=head1 AUTHOR

Matti Lattu (matti.lattu@helsinki.fi)

=head1 LICENSE AND COPYRIGHT

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.1 or,
at your option, any later version of Perl 5 you may have available.

Copyright (C) 2012 University of Helsinki

